// ****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<com.ibm.etools.cobol>>
//   (C) Copyright IBM Corp. 2001,2004 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************

package com.ibm.etools.cobol.plugin;

/**
 * @author: Mahendra Akkina
 */
import java.net.URL;
import java.text.MessageFormat;
import java.util.Date;
import java.util.Hashtable;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import com.ibm.etools.logging.util.DefaultRecord;
import com.ibm.etools.logging.util.IPluginHelper;
import com.ibm.etools.logging.util.Level;
import com.ibm.etools.logging.util.MsgLogger;
import com.ibm.etools.logging.util.PluginHelperImpl;

public class CobolPlugin extends AbstractUIPlugin implements IPluginHelper {
	private static final String copyright = "(c) Copyright IBM Corporation 2001,2004.";
	private static final String EXTENSION_POINT_ID = "cobolImporterCopybookLocation";	

	  /**
	  * Use this severity to log any event associated with
	  * a task that has succeeded.
	  */
	  public static final int OK = 0;

	  /**
	  * Use this severity to log any event associated with
	  * a task that has succeeded but with information the
	  * user may find useful.
	  */
	  public static final int INFO = 1;

	  /**
	  * Use this severity to log any event associated with
	  * a task that has succeeded but with side effects
	  * that may surprise the user.
	  */
	  public static final int WARNING = 2;

	  /**
	  * Use this severity to log any event associated with
	  * a task that has failed.
	  */
	  public static final int ERROR = 4;
	
	/*
  	 * Plugin.xml constants
  	 */
   	public static final String PLUGIN_ID="com.ibm.etools.cobol";//$NON-NLS-1$
   
  	/**
  	 * Keep track of the singleton.
  	 */
  	private static CobolPlugin fieldPlugin;

  	/**
  	 * Logger helper.
  	 */
	protected static MsgLogger msgLogger; // Logger that does real Tracing
	private static String additionalCOPYBookLocations = "";


/**
 * CobolPreferencePagePlugin constructor comment.
 * Creates the plugin and cache its default instance
 */
  public CobolPlugin() {

	super();
		
	// Remember the static instance.
	if (fieldPlugin==null)
		fieldPlugin=this;   
}
/**
 * Returns the plugin singleton.
 */
static public CobolPlugin getDefault() {
	return fieldPlugin;
}

  /**
   * Get the singleton instance.
   */
  public static CobolPlugin getPlugin()
  {
	return fieldPlugin;
  }
  
  /**
   * This method is called upon plug-in activation
   */
  public void start(BundleContext context) throws Exception {
  	super.start(context);
  	IExtensionPoint point = Platform.getExtensionRegistry().getExtensionPoint(PLUGIN_ID,EXTENSION_POINT_ID);
  	//check: Any <extension> tags for our extension-point?
  	if (point !=null)
  	{
  		IExtension[] extensions = point.getExtensions();
  		for (int i =0;i <extensions.length;i++)	{
  			IConfigurationElement[] ces = extensions[i].getConfigurationElements();
  			for (int j =0;j <ces.length;j++){
  				if ((ces[j].getAttribute("plugin-id") != "") && (ces[j].getAttribute("relativeFolderLocation") != null)) {
  					String plugin_id = ces[j].getAttribute("plugin-id");
  					String relativeFolderLocation = ces[j].getAttribute("relativeFolderLocation");
  					IPath path = new Path(CobolPlugin.getPluginLocation(plugin_id) + relativeFolderLocation);
  					if (path.toFile().exists()){
  						additionalCOPYBookLocations = additionalCOPYBookLocations + CobolPlugin.getPluginLocation(plugin_id) + relativeFolderLocation + ";";  							
  					} else {
  						CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolPlugin.start(): Unable to add the additionalCopyBookLocation: " + CobolPlugin.getPluginLocation(plugin_id) + relativeFolderLocation + " does not exist");
  					}
  				}
  			}
  		}
  	}		
  }
  
/**
 * Returns the workbench used by the example
 *
 * This method is internal to the plugin and must not be called
 * by any other plugins.
 */
public IWorkbench getWorkbench() {
	return PlatformUI.getWorkbench();
}

public void setMsgLoggerConfig(Hashtable msgLoggerConfig){
	getMsgLogger().setMsgLoggerConfig(msgLoggerConfig);
}

public Hashtable getMsgLoggerConfig(Plugin plugin) {
    return (new PluginHelperImpl().getMsgLoggerConfig(plugin));
}

public Hashtable getMsgLoggerConfig() {
    return (getMsgLoggerConfig(this));
}

/**
 * This write method takes a logLevel parameter, look in
 * com.ibm.etools.logging.util.Level for the defaults
 * available. According to logLevel parameter specified
 * the message is being written to log file
 * log file.
 * @param logLevel int
 * @param msg java.lang.String
 */
public void writeMsg(int logLevel, String msg) {
	writeMsg(logLevel, msg, null);	
}
/**
 * This write method takes a logLevel parameter, look in
 * com.ibm.etools.logging.util.Level for the defaults
 * available. According to logLevel parameter specified
 * the message is being written to the log file
 * @param logLevel int
 * @param msg java.lang.String
 */
public void writeMsg(int logLevel, String msg, Throwable t) {
	Date date = new Date();
	getMsgLogger().write(logLevel, date.toString() + msg);
	if (t != null)
		getMsgLogger().write(logLevel, t);
}

/**
 * @see com.ibm.etools.ctc.plugin.BasePlugin
 */
public void initialize() throws org.eclipse.core.runtime.CoreException {}
protected void initializeDefaultPreferences(IPreferenceStore store){
	
	// set preferences
	store.setDefault(ICobolPluginConstants.PLUGIN_ID,"com.ibm.etools.cobol.plugin");

	// Preference constants
	store.setDefault(ICobolPluginConstants.PRE_TOOL_EDITOR,false);
	store.setDefault(ICobolPluginConstants.PRE_TOOL_NEW_WIZARD,false);
	store.setDefault(ICobolPluginConstants.PRE_TOOL_PREFERENCE_PAGE,true);
	store.setDefault(ICobolPluginConstants.PRE_TOOL_EXTENSION,true);
	store.setDefault(ICobolPluginConstants.PRE_TOOL_BASE_ID,"CobolPlugin");
	store.setDefault(ICobolPluginConstants.PRE_TOOL_VENDOR,"IBM");
	store.setDefault(ICobolPluginConstants.PRE_COBOL_COMPILE_TRUNC_BIN_OPTION, "true");
}

/**
 * Returns the translated String found with the given key.
 *
 * @param key java.lang.String
 * @return java.lang.String
 */
public static String getResource(String key) {
	try {
		return Platform.getResourceString(getDefault().getBundle(), key);
	} catch (Exception e) {
		return key;
	}
}
/**
 * Returns the translated String found with the given key,
 * and formatted with the given arguments using java.text.MessageFormat.
 *
 * @param key java.lang.String
 * @param arguments java.lang.Object[]
 * @return java.lang.String
 */
public static String getResource(String key, Object[] arguments) {
	try {
		String text = getResource(key);
		return MessageFormat.format(text, arguments);
	} catch (Exception e) {
		return key;
	}
}

// To Provide Tracing Facility
public static MsgLogger getMsgLogger() {
	if (msgLogger == null) {
		msgLogger =
			(MsgLogger) MsgLogger.getFactory().getLogger(
				PluginHelperImpl.getMsgLoggerName(fieldPlugin),
				fieldPlugin);
	}
	return msgLogger;
}

/**
 * Writes an exception object to the log file.
 * @param msgSource The object (ie. <code>this</code>) writing the log entry.
 * @param methodName The name of the method writing the log entry.
 * @param severity The severity, one of
 * {@link #OK OK},
 * {@link #INFO INFO},
 * {@link #WARNING WARNING} or
 * {@link #ERROR ERROR}.
 * @param msgKey The key used to look up the message in plugin.properties.
 */
public static void write(Object msgSource, String methodName, int severity, String msgKey) {
	DefaultRecord aDefaultRecord = new DefaultRecord (getDefault().getPlugin().getDescriptor().getUniqueIdentifier(),
			msgSource, methodName, severity, getResource(msgKey));
	msgLogger.write(severity, aDefaultRecord);
}

	public static String getAdditionalCOPYBookLocations() {
		return additionalCOPYBookLocations;
	}	

	public static String getPluginLocation(String pluginId) {
		try {
			URL installURL = Platform.getBundle(pluginId).getEntry("/");
			return Platform.resolve(installURL).getFile().substring(1);
		}
		catch (Exception e) {
			return null;
		}
	} 	
}
